/*
-- XMLScriptEngine - library to run XML scripts in some AS3 environment
-- Copyright (C) 2007 Janvarev Vladislav
-- http://janvarev.ru/XMLScriptEngine
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published
-- by the Free Software Foundation; either version 2.1 of the License, or
-- (at your option) any later version.
--
-- This library is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-- Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public
-- License along with this library; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package script
{
	import mx.controls.Alert;
	import mx.core.Application;
	import flash.events.EventDispatcher;
	
	public class XMLScriptEngine extends EventDispatcher
	{
		// Singleton
		
		private static var _instance: XMLScriptEngine = null;
		
		public static function getInstance():XMLScriptEngine {
			if(_instance == null) {
				_instance = new XMLScriptEngine();
			}
			return _instance;
		}
		
		public function XMLScriptEngine() {
			//	
		}

		public var externalFunctionsHandlers:Object = {};
		
		
		// 1 - run all
		// 2 - run without objects
		public var securityLevel:int = 2;
		
		// making commands
		
		public function cmdSetParam(objId:String, paramName: String, paramValue: Object):XML {
			var cmdXML:XML = <sp />;
			if(objId!="") {
				cmdXML.@id = objId;
			}
			cmdXML.@n = paramName;
			cmdXML.@v = paramValue;
			return cmdXML;
		}
		
		// running commands
	
		public function runCommands(defaultObject:Object,environment:Object,commands:XML):Object {
			if(commands == null) return null;
			
			var cmd:XML;
			var result:Object;
			for each(cmd in commands.children()) {
				result = runCommand(defaultObject,environment,cmd);
			}
			return result;			
		}
		
		// run one instruction
		
		public function runCommand(defaultObject:Object,environment:Object,cmd:XML):Object {
			var obj:Object;
			var tempValue: Object;
			
			obj = getObject(defaultObject,environment,cmd);
			
			// if applyable object not found - throw an error
			if(obj==null) throw new Error("XMLScriptEngine: can't find "+cmd.@id + " in environment");
			
			// set property - <sp id="global" n="width" v="600 /> --> 600
			if(cmd.name() == "sp") {
				tempValue = getParam(defaultObject,environment,cmd,"v");
				obj[cmd.@n] = tempValue;
				return tempValue;
			}
			
			// get property - <gp id="global" n="width" /> --> 600
			if(cmd.name() == "gp") {
				return obj[cmd.@n];
			}
			
			// math add <madd p1="600" p2="200" /> --> 800
			if(cmd.name() == "madd") {
				return Number(getParam(defaultObject,environment,cmd,"p1")) + Number(getParam(defaultObject,environment,cmd,"p2"));
			}

			// math sub <msub p1="600" p2="200" /> --> 400
			if(cmd.name() == "msub") {
				return Number(getParam(defaultObject,environment,cmd,"p1")) - Number(getParam(defaultObject,environment,cmd,"p2"));
			}

			// if condition <if cond="true"><then>..<gp id="global" n="width"></then><else>..</else></if> --> 600 (getting last result)
			if(cmd.name() == "if") {
				tempValue = getParam(defaultObject,environment,cmd,"cond");
				if(Boolean(tempValue)) {
					return runCommands(defaultObject,environment,cmd["then"][0]);
				} else {
					return runCommands(defaultObject,environment,cmd["else"][0]);
				}
			}
			
			// while loop  <while><cond>..</cond><do>..</do></while>
			if(cmd.name() == "while") {
				tempValue = getParam(defaultObject,environment,cmd,"cond");
				while (Boolean(tempValue)) {
					runCommands(defaultObject,environment,cmd["do"][0]);
				}
				return null;
			}
			
			// bool functions
			if(cmd.name() == "eq") {
				return getParam(defaultObject,environment,cmd,"p1") == getParam(defaultObject,environment,cmd,"p2");
			}
			
			if(cmd.name() == "not") {
				return !getParam(defaultObject,environment,cmd,"p1");
			}
			
			if(cmd.name() == "neq") {
				return getParam(defaultObject,environment,cmd,"p1") == getParam(defaultObject,environment,cmd,"p2");
			}
			
			if(cmd.name() == "gt") {
				return getParam(defaultObject,environment,cmd,"p1") > getParam(defaultObject,environment,cmd,"p2");
			}
			
			if(cmd.name() == "lt") {
				return getParam(defaultObject,environment,cmd,"p1") < getParam(defaultObject,environment,cmd,"p2");
			}
			
			if(cmd.name() == "and") {
				return getParam(defaultObject,environment,cmd,"p1") && getParam(defaultObject,environment,cmd,"p2");
			}
			
			if(cmd.name() == "or") {
				return getParam(defaultObject,environment,cmd,"p1") || getParam(defaultObject,environment,cmd,"p2");
			}
			
			// run functions with different number of parameters --> result of function, if have
			if(cmd.name() == "run0") {
				return obj[cmd.@n]();
			}
			if(cmd.name() == "run1") {
				return obj[cmd.@n](getParam(defaultObject,environment,cmd,"p1"));
			}
			if(cmd.name() == "run2") {
				return obj[cmd.@n](getParam(defaultObject,environment,cmd,"p1"),getParam(defaultObject,environment,cmd,"p2"));
			}
			if(cmd.name() == "run3") {
				return obj[cmd.@n](getParam(defaultObject,environment,cmd,"p1"),
							getParam(defaultObject,environment,cmd,"p2"),
							getParam(defaultObject,environment,cmd,"p3"));
			}
			
			// running external functions <extFunc funcName="showMessage" msg="Hello, world!" /> -> result of function, if have
			if(cmd.name() == "extFunc") {
				if(externalFunctionsHandlers[cmd.@funcName]!=null) {
					return externalFunctionsHandlers[cmd.@funcName](defaultObject,environment,cmd);
				}
			}
			
			// environment management -> add a new object in environment with id @envId
			if(cmd.name() == "makeObject") {
				environment[cmd.@envId] = new Object;	
			}
			
			// --------------- util functions ---------------
			// <spadd n="x" v="4" /> -> x=x+4
			if(cmd.name() == "spadd") {
				tempValue = getParam(defaultObject,environment,cmd,"v");
				obj[cmd.@n] += Number(tempValue);
				return tempValue;
			}
			
			// <spsub n="x" v="4" /> ---> x=x-4
			if(cmd.name() == "spsub") {
				tempValue = getParam(defaultObject,environment,cmd,"v");
				obj[cmd.@n] -= Number(tempValue);
				return tempValue;
			}
			
			// advanced and powerful functions - run only if you trust content
			if(securityLevel < 2) {
				if(cmd.name() == "setObject") {
					environment[cmd.@envId] = getParam(defaultObject,environment,cmd,"v");
				}
			}
			
			return null;
		}
		
		// return value of some parameter in command - if needed, run other commands
		
		public function getParam(defaultObject:Object,environment:Object,cmd:XML,paramName:String):Object {
			if(cmd.hasOwnProperty("@"+paramName)) {
				return cmd["@"+paramName];
			}
			return runCommands(defaultObject,environment,cmd[paramName][0]);
			
		}
		
		// find applyable object in command
		
		public function getObject(defaultObject:Object,environment:Object,cmd:XML):Object {
			if(!cmd.hasOwnProperty("@id")) {
				return defaultObject;
			} else {
				return environment[cmd.@id];
			}
		}
		
		// additional functions management
		
		public function addExternalFunction(funcName:String, func:Function):void {
			externalFunctionsHandlers[funcName] = func;		
		}
	}
}